@@ -9,17 +9,15 @@
 mod chunk_storage;
 
 use crate::{
-    error::convert_to_error_message,
-    node_ops::{MsgType, NodeDuties, NodeDuty, OutgoingMsg},
+    node_ops::{NodeDuties, NodeDuty},
     Result,
 };
 use chunk_storage::ChunkStorage;
 use log::info;
-use sn_data_types::{Blob, BlobAddress};
+use sn_data_types::{Blob, BlobAddress, PublicKey};
 use sn_messaging::{
-    client::{BlobRead, BlobWrite, CmdError},
-    node::{NodeEvent, NodeMsg},
-    Aggregation, DstLocation, EndUser, MessageId,
+    client::{BlobRead, BlobWrite},
+    MessageId,
 };
 use std::{
     fmt::{self, Display, Formatter},
@@ -62,13 +60,12 @@ impl Chunks {
         &mut self,
         write: &BlobWrite,
         msg_id: MessageId,
-        origin: EndUser,
+        requester: PublicKey,
     ) -> Result<NodeDuty> {
         match &write {
-            BlobWrite::New(data) => self.chunk_storage.store(&data, msg_id).await,
-            // really though, for a delete, what we should be looking at is the origin signature! That would be the source of truth!
+            BlobWrite::New(data) => self.chunk_storage.store(&data).await,
             BlobWrite::DeletePrivate(address) => {
-                self.chunk_storage.delete(*address, msg_id, origin).await
+                self.chunk_storage.delete(*address, msg_id, requester).await
             }
         }
     }
@@ -83,27 +80,9 @@ impl Chunks {
     }
 
     /// Stores a chunk that Elders sent to it for replication.
-    pub async fn store_for_replication(
-        &mut self,
-        blob: Blob,
-        msg_id: MessageId,
-    ) -> Result<NodeDuty> {
-        let data_name = *blob.address().name();
-        let result = match self.chunk_storage.store_for_replication(blob).await {
-            Ok(()) => Ok(()),
-            Err(err) => Err(CmdError::Data(convert_to_error_message(err))),
-        };
-        Ok(NodeDuty::Send(OutgoingMsg {
-            msg: MsgType::Node(NodeMsg::NodeEvent {
-                event: NodeEvent::ChunkWriteHandled(result),
-                id: MessageId::in_response_to(&msg_id),
-                correlation_id: msg_id,
-            }),
-            section_source: false, // sent as single node
-            // Data's metadata section
-            dst: DstLocation::Section(data_name),
-            aggregation: Aggregation::None,
-        }))
+    pub async fn store_for_replication(&mut self, blob: Blob) -> Result<NodeDuty> {
+        self.chunk_storage.store_for_replication(blob).await?;
+        Ok(NodeDuty::NoOp)
     }
 }
 
