@@ -12,10 +12,10 @@ use crate::{
     node_ops::{NodeDuties, NodeDuty},
     Error, Result,
 };
-use log::{debug, error, info, warn};
+use log::{debug, info, warn};
 use sn_data_types::{Blob, BlobAddress, PublicKey};
 use sn_messaging::{
-    client::{BlobDataExchange, BlobRead, BlobWrite, CmdError, QueryResponse},
+    client::{BlobDataExchange, BlobRead, BlobWrite, ClientSigned, CmdError, QueryResponse},
     node::{NodeCmd, NodeMsg, NodeQuery, NodeSystemCmd},
     Aggregation, EndUser, MessageId,
 };
@@ -103,12 +103,13 @@ impl BlobRecords {
         &mut self,
         write: BlobWrite,
         msg_id: MessageId,
+        client_signed: ClientSigned,
         origin: EndUser,
     ) -> Result<NodeDuty> {
         use BlobWrite::*;
         match write {
-            New(data) => self.store(data, msg_id, origin).await,
-            DeletePrivate(address) => self.delete(address, msg_id, origin).await,
+            New(data) => self.store(data, msg_id, client_signed, origin).await,
+            DeletePrivate(address) => self.delete(address, msg_id, client_signed, origin).await,
         }
     }
 
@@ -157,6 +158,7 @@ impl BlobRecords {
         &mut self,
         data: Blob,
         msg_id: MessageId,
+        client_signed: ClientSigned,
         origin: EndUser,
     ) -> Result<NodeDuty> {
         let target_holders = self
@@ -178,81 +180,35 @@ impl BlobRecords {
                 .await;
         }
 
-        let blob_address = *data.address();
         let blob_write = BlobWrite::New(data);
 
-        if self
-            .adult_liveness
-            .new_write(msg_id, Some(origin), blob_address, &target_holders)
-        {
-            Ok(NodeDuty::SendToNodes {
-                targets: target_holders,
-                msg: NodeMsg::NodeCmd {
-                    cmd: NodeCmd::Chunks {
-                        cmd: blob_write,
-                        origin,
-                    },
-                    id: msg_id,
+        Ok(NodeDuty::SendToNodes {
+            targets: target_holders,
+            msg: NodeMsg::NodeCmd {
+                cmd: NodeCmd::Chunks {
+                    cmd: blob_write,
+                    client_signed,
+                    origin,
                 },
-                aggregation: Aggregation::AtDestination,
-            })
-        } else {
-            info!(
-                "Operation with MessageId {:?} is already in progress",
-                msg_id
-            );
-            Ok(NodeDuty::NoOp)
-        }
+                id: msg_id,
+            },
+            aggregation: Aggregation::AtDestination,
+        })
     }
 
-    async fn store(&mut self, data: Blob, msg_id: MessageId, origin: EndUser) -> Result<NodeDuty> {
-        if let Err(error) = validate_data_owner(&data, &origin) {
+    async fn store(
+        &mut self,
+        data: Blob,
+        msg_id: MessageId,
+        client_signed: ClientSigned,
+        origin: EndUser,
+    ) -> Result<NodeDuty> {
+        if let Err(error) = validate_data_owner(&data, &client_signed.public_key) {
             return self.send_error(error, msg_id, origin).await;
         }
 
-        self.send_chunks_to_adults(data, msg_id, origin).await
-    }
-
-    pub async fn record_adult_write_liveness(
-        &mut self,
-        correlation_id: MessageId,
-        result: Result<(), CmdError>,
-        src: XorName,
-    ) -> NodeDuties {
-        let mut duties = vec![];
-        if let Some((address, origin)) = self
-            .adult_liveness
-            .record_adult_write_liveness(correlation_id, src)
-        {
-            if let Err(error) = result {
-                error!("Error at Adult while performing a BlobWrite: {:?}", &error);
-                // Depending on error, we might have to take action here.
-                if let Some(end_user) = origin {
-                    duties.push(NodeDuty::Send(build_client_error_response(
-                        error,
-                        correlation_id,
-                        end_user,
-                    )))
-                }
-            } else {
-                info!(
-                    "AdultWrite operation at {:?} MessageId {:?} at {:?} was successful",
-                    address, correlation_id, src
-                );
-            }
-        }
-        let mut unresponsive_adults = Vec::new();
-        for (name, count) in self.adult_liveness.find_unresponsive_adults() {
-            warn!(
-                "Adult {} has {} pending ops. It might be unresponsive",
-                name, count
-            );
-            unresponsive_adults.push(name);
-        }
-        if !unresponsive_adults.is_empty() {
-            duties.push(NodeDuty::ProposeOffline(unresponsive_adults))
-        }
-        duties
+        self.send_chunks_to_adults(data, msg_id, client_signed, origin)
+            .await
     }
 
     pub async fn record_adult_read_liveness(
@@ -268,10 +224,11 @@ impl BlobRecords {
             )));
         }
         let mut duties = vec![];
-        if let Some((_address, end_user)) = self
-            .adult_liveness
-            .record_adult_read_liveness(correlation_id, src)
-        {
+        if let Some((_address, end_user)) = self.adult_liveness.record_adult_read_liveness(
+            correlation_id,
+            src,
+            response.is_success(),
+        ) {
             return Ok(vec![NodeDuty::Send(build_client_query_response(
                 response,
                 correlation_id,
@@ -310,34 +267,25 @@ impl BlobRecords {
         &mut self,
         address: BlobAddress,
         msg_id: MessageId,
+        client_signed: ClientSigned,
         origin: EndUser,
     ) -> Result<NodeDuty> {
         let targets = self.get_holders_for_chunk(address.name()).await;
         let targets = targets.iter().cloned().collect::<BTreeSet<_>>();
 
-        if self
-            .adult_liveness
-            .new_write(msg_id, Some(origin), address, &targets)
-        {
-            let msg = NodeMsg::NodeCmd {
-                cmd: NodeCmd::Chunks {
-                    cmd: BlobWrite::DeletePrivate(address),
-                    origin,
-                },
-                id: msg_id,
-            };
-            Ok(NodeDuty::SendToNodes {
-                msg,
-                targets,
-                aggregation: Aggregation::AtDestination,
-            })
-        } else {
-            info!(
-                "Operation with MessageId {:?} is already in progress",
-                msg_id
-            );
-            Ok(NodeDuty::NoOp)
-        }
+        let msg = NodeMsg::NodeCmd {
+            cmd: NodeCmd::Chunks {
+                cmd: BlobWrite::DeletePrivate(address),
+                client_signed,
+                origin,
+            },
+            id: msg_id,
+        };
+        Ok(NodeDuty::SendToNodes {
+            msg,
+            targets,
+            aggregation: Aggregation::AtDestination,
+        })
     }
 
     pub(super) async fn republish_chunk(&mut self, data: Blob) -> Result<NodeDuty> {
@@ -359,22 +307,14 @@ impl BlobRecords {
             msg_id
         );
 
-        if self
-            .adult_liveness
-            .new_write(msg_id, None, *data.address(), &target_holders)
-        {
-            Ok(NodeDuty::SendToNodes {
-                targets: target_holders,
-                msg: NodeMsg::NodeCmd {
-                    cmd: NodeCmd::System(NodeSystemCmd::ReplicateChunk(data)),
-                    id: msg_id,
-                },
-                aggregation: Aggregation::None,
-            })
-        } else {
-            info!("Skipping chunk republish since it's already in progress");
-            Ok(NodeDuty::NoOp)
-        }
+        Ok(NodeDuty::SendToNodes {
+            targets: target_holders,
+            msg: NodeMsg::NodeCmd {
+                cmd: NodeCmd::System(NodeSystemCmd::ReplicateChunk(data)),
+                id: msg_id,
+            },
+            aggregation: Aggregation::None,
+        })
     }
 
     pub(super) async fn read(
@@ -444,13 +384,13 @@ impl BlobRecords {
     }
 }
 
-fn validate_data_owner(data: &Blob, origin: &EndUser) -> Result<()> {
+fn validate_data_owner(data: &Blob, requester: &PublicKey) -> Result<()> {
     if data.is_private() {
         data.owner()
-            .ok_or_else(|| Error::InvalidOwners(*origin.id()))
+            .ok_or_else(|| Error::InvalidOwners(*requester))
             .and_then(|data_owner| {
-                if data_owner != origin.id() {
-                    Err(Error::InvalidOwners(*origin.id()))
+                if data_owner != requester {
+                    Err(Error::InvalidOwners(*requester))
                 } else {
                     Ok(())
                 }
